// Copyright (C) 1996 by Florian Raemy & LCAV, Swiss Federal Institute of 
// Technology, Lausanne, Switzerland.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit Florian's Home Page http://studwww.eurecom.fr/~raemy and
// LCAV Department of EPFL: http://lcavwww.epfl.ch

import java.lang.*;
import java.util.*;
import java.io.*;


 /*
 * DCT_YUV - A Java implementation of the Discreet Cosine Transform for YUV Components
 */

public class DCT_YUV
{
    /**
     * DCT Block Size - default 8
     */
    public int N        = 8;

    /**
     * Image Quality (0-25) - default 1 (worst image / best compression)
     */
    public int QUALITY ;

    /**
     * Image width - must correspond to imageArray bounds
     */
    public int ROWS   ;

    /**
     * Image height - must correspond to imageArray bounds
     */
    public int COLS  ;

    /**
     * The ZigZag matrix.
     */
    public int zigZag[][] = new int[64][2];

    /**
     * Cosine matrix. N * N.
     */
    public double c[][]        = new double[N][N];

    /**
     * Transformed cosine matrix, N*N.
     */
    public double cT[][]       = new double[N][N];

    /**
     * Quantitization Matrix for luminace.
     */
    public float quantum_luminance[][]     = new float[N][N];

    /**
     * Quantitization Matrix for chrominance.
     */
    public float quantum_chrominance[][]     = new float[N][N];

    /**
     * DCT Result Matrix
     */
    public int resultDCT[][] = new int[ROWS][COLS];

    /**
     * Constructs a new DCT object. Initializes the cosine transform matrix
     * these are used when computing the DCT and it's inverse. This also
     * initializes the run length counters and the ZigZag sequence. Note that
     * the image quality can be worse than 25 however the image will be
     * extemely pixelated, usually to a block size of N.
     *
     * @param QUALITY The quality of the image (0 best - 25 worst)
     *
     */
    public DCT_YUV(int QUALITY, int ROW, int COL)
    {
        initZigZag();
        initMatrix(QUALITY);
        ROWS=ROW;
        COLS=COL;
    }


    /**
     * This method sets up the quantization matrix for luminance and chrominance
     * using the Quality parameter,
     * and then sets up the Cosine Transform Matrix and the Transposed CT.
     * These are used by the forward and inverse DCT. The RLE encoding
     * variables are set up to track the number of consecutive zero values
     * that have output or will be input.
     * @param quality The quality scaling factor
     */
    private void initMatrix(int quality)
    {
        int i;
        int j;

        /*
         * INITIALISATION OF THE LUMINANCE MATRIX
         */

        quantum_luminance[0][0]=quality*16;
        quantum_luminance[0][1]=quality*11;
        quantum_luminance[0][2]=quality*10;
        quantum_luminance[0][3]=quality*16;
        quantum_luminance[0][4]=quality*24;
        quantum_luminance[0][5]=quality*40;
        quantum_luminance[0][6]=quality*51;
        quantum_luminance[0][7]=quality*61;
        quantum_luminance[1][0]=quality*12;
        quantum_luminance[1][1]=quality*12;
        quantum_luminance[1][2]=quality*14;
        quantum_luminance[1][3]=quality*19;
        quantum_luminance[1][4]=quality*26;
        quantum_luminance[1][5]=quality*58;
        quantum_luminance[1][6]=quality*60;
        quantum_luminance[1][7]=quality*55;
        quantum_luminance[2][0]=quality*14;
        quantum_luminance[2][1]=quality*13;
        quantum_luminance[2][2]=quality*16;
        quantum_luminance[2][3]=quality*24;
        quantum_luminance[2][4]=quality*40;
        quantum_luminance[2][5]=quality*57;
        quantum_luminance[2][6]=quality*69;
        quantum_luminance[2][7]=quality*56;
        quantum_luminance[3][0]=quality*14;
        quantum_luminance[3][1]=quality*17;
        quantum_luminance[3][2]=quality*22;
        quantum_luminance[3][3]=quality*29;
        quantum_luminance[3][4]=quality*51;
        quantum_luminance[3][5]=quality*87;
        quantum_luminance[3][6]=quality*80;
        quantum_luminance[3][7]=quality*62;
        quantum_luminance[4][0]=quality*18;
        quantum_luminance[4][1]=quality*22;
        quantum_luminance[4][2]=quality*37;
        quantum_luminance[4][3]=quality*56;
        quantum_luminance[4][4]=quality*68;
        quantum_luminance[4][5]=quality*109;
        quantum_luminance[4][6]=quality*103;
        quantum_luminance[4][7]=quality*77;
        quantum_luminance[5][0]=quality*24;
        quantum_luminance[5][1]=quality*35;
        quantum_luminance[5][2]=quality*55;
        quantum_luminance[5][3]=quality*64;
        quantum_luminance[5][4]=quality*81;
        quantum_luminance[5][5]=quality*104;
        quantum_luminance[5][6]=quality*113;
        quantum_luminance[5][7]=quality*92;
        quantum_luminance[6][0]=quality*49;
        quantum_luminance[6][1]=quality*64;
        quantum_luminance[6][2]=quality*78;
        quantum_luminance[6][3]=quality*87;
        quantum_luminance[6][4]=quality*103;
        quantum_luminance[6][5]=quality*121;
        quantum_luminance[6][6]=quality*120;
        quantum_luminance[6][7]=quality*101;
        quantum_luminance[7][0]=quality*72;
        quantum_luminance[7][1]=quality*92;
        quantum_luminance[7][2]=quality*95;
        quantum_luminance[7][3]=quality*98;
        quantum_luminance[7][4]=quality*112;
        quantum_luminance[7][5]=quality*100;
        quantum_luminance[7][6]=quality*103;
        quantum_luminance[7][7]=quality*99;

        /*
         * INITIALISATION OF CHROMINANCE MATRIX
         */
        quantum_chrominance[0][0]=quality*17;
        quantum_chrominance[0][1]=quality*18;
        quantum_chrominance[0][2]=quality*24;
        quantum_chrominance[0][3]=quality*47;
        quantum_chrominance[0][4]=quality*99;
        quantum_chrominance[0][5]=quality*99;
        quantum_chrominance[0][6]=quality*99;
        quantum_chrominance[0][7]=quality*99;
        quantum_chrominance[1][0]=quality*18;
        quantum_chrominance[1][1]=quality*21;
        quantum_chrominance[1][2]=quality*26;
        quantum_chrominance[1][3]=quality*66;
        quantum_chrominance[1][4]=quality*99;
        quantum_chrominance[1][5]=quality*99;
        quantum_chrominance[1][6]=quality*99;
        quantum_chrominance[1][7]=quality*99;
        quantum_chrominance[2][0]=quality*24;
        quantum_chrominance[2][1]=quality*26;
        quantum_chrominance[2][2]=quality*56;
        quantum_chrominance[2][3]=quality*99;
        quantum_chrominance[2][4]=quality*99;
        quantum_chrominance[2][5]=quality*99;
        quantum_chrominance[2][6]=quality*99;
        quantum_chrominance[2][7]=quality*99;
        quantum_chrominance[3][0]=quality*47;
        quantum_chrominance[3][1]=quality*66;
        quantum_chrominance[3][2]=quality*99;
        quantum_chrominance[3][3]=quality*99;
        quantum_chrominance[3][4]=quality*99;
        quantum_chrominance[3][5]=quality*99;
        quantum_chrominance[3][6]=quality*99;
        quantum_chrominance[3][7]=quality*99;
        quantum_chrominance[4][0]=quality*99;
        quantum_chrominance[4][1]=quality*99;
        quantum_chrominance[4][2]=quality*99;
        quantum_chrominance[4][3]=quality*99;
        quantum_chrominance[4][4]=quality*99;
        quantum_chrominance[4][5]=quality*99;
        quantum_chrominance[4][6]=quality*99;
        quantum_chrominance[4][7]=quality*99;
        quantum_chrominance[5][0]=quality*99;
        quantum_chrominance[5][1]=quality*99;
        quantum_chrominance[5][2]=quality*99;
        quantum_chrominance[5][3]=quality*99;
        quantum_chrominance[5][4]=quality*99;
        quantum_chrominance[5][5]=quality*99;
        quantum_chrominance[5][6]=quality*99;
        quantum_chrominance[5][7]=quality*99;
        quantum_chrominance[6][0]=quality*99;
        quantum_chrominance[6][1]=quality*99;
        quantum_chrominance[6][2]=quality*99;
        quantum_chrominance[6][3]=quality*99;
        quantum_chrominance[6][4]=quality*99;
        quantum_chrominance[6][5]=quality*99;
        quantum_chrominance[6][6]=quality*99;
        quantum_chrominance[6][7]=quality*99;
        quantum_chrominance[7][0]=quality*99;
        quantum_chrominance[7][1]=quality*99;
        quantum_chrominance[7][2]=quality*99;
        quantum_chrominance[7][3]=quality*99;
        quantum_chrominance[7][4]=quality*99;
        quantum_chrominance[7][5]=quality*99;
        quantum_chrominance[7][6]=quality*99;
        quantum_chrominance[7][7]=quality*99;


        /*
         * INITIALISATION OF THE C & CT MATRIX FOR DCT
         */
        for (j = 0; j < N; j++)
        {
            double nn = (double)(N);
            c[0][j]  = 1.0 / Math.sqrt(nn);
            cT[j][0] = c[0][j];
        }
        for (i = 1; i < 8; i++)
        {
            for (j = 0; j < 8; j++)
            {
                double jj = (double)j;
                double ii = (double)i;
                c[i][j]  = Math.sqrt(2.0/8.0) * Math.cos(((2.0 * jj + 1.0) * ii * Math.PI) / (2.0 * 8.0));
                cT[j][i] = c[i][j];
            }
        }
    }

    /**
     * Initializes the ZigZag matrix.
     */
    private void initZigZag()
    {
        zigZag[0][0] = 0; // 0,0
        zigZag[0][1] = 0;
        zigZag[1][0] = 0; // 0,1
        zigZag[1][1] = 1;
        zigZag[2][0] = 1; // 1,0
        zigZag[2][1] = 0;
        zigZag[3][0] = 2; // 2,0
        zigZag[3][1] = 0;
        zigZag[4][0] = 1; // 1,1
        zigZag[4][1] = 1;
        zigZag[5][0] = 0; // 0,2
        zigZag[5][1] = 2;
        zigZag[6][0] = 0; // 0,3
        zigZag[6][1] = 3;
        zigZag[7][0] = 1; // 1,2
        zigZag[7][1] = 2;
        zigZag[8][0] = 2; // 2,1
        zigZag[8][1] = 1;
        zigZag[9][0] = 3; // 3,0
        zigZag[9][1] = 0;
        zigZag[10][0] = 4; // 4,0
        zigZag[10][1] = 0;
        zigZag[11][0] = 3; // 3,1
        zigZag[11][1] = 1;
        zigZag[12][0] = 2; // 2,2
        zigZag[12][1] = 2;
        zigZag[13][0] = 1; // 1,3
        zigZag[13][1] = 3;
        zigZag[14][0] = 0; // 0,4
        zigZag[14][1] = 4;
        zigZag[15][0] = 0; // 0,5
        zigZag[15][1] = 5;
        zigZag[16][0] = 1; // 1,4
        zigZag[16][1] = 4;
        zigZag[17][0] = 2; // 2,3
        zigZag[17][1] = 3;
        zigZag[18][0] = 3; // 3,2
        zigZag[18][1] = 2;
        zigZag[19][0] = 4; // 4,1
        zigZag[19][1] = 1;
        zigZag[20][0] = 5; // 5,0
        zigZag[20][1] = 0;
        zigZag[21][0] = 6; // 6,0
        zigZag[21][1] = 0;
        zigZag[22][0] = 5; // 5,1
        zigZag[22][1] = 1;
        zigZag[23][0] = 4; // 4,2
        zigZag[23][1] = 2;
        zigZag[24][0] = 3; // 3,3
        zigZag[24][1] = 3;
        zigZag[25][0] = 2; // 2,4
        zigZag[25][1] = 4;
        zigZag[26][0] = 1; // 1,5
        zigZag[26][1] = 5;
        zigZag[27][0] = 0; // 0,6
        zigZag[27][1] = 6;
        zigZag[28][0] = 0; // 0,7
        zigZag[28][1] = 7;
        zigZag[29][0] = 1; // 1,6
        zigZag[29][1] = 6;
        zigZag[30][0] = 2; // 2,5
        zigZag[30][1] = 5;
        zigZag[31][0] = 3; // 3,4
        zigZag[31][1] = 4;
        zigZag[32][0] = 4; // 4,3
        zigZag[32][1] = 3;
        zigZag[33][0] = 5; // 5,2
        zigZag[33][1] = 2;
        zigZag[34][0] = 6; // 6,1
        zigZag[34][1] = 1;
        zigZag[35][0] = 7; // 7,0
        zigZag[35][1] = 0;
        zigZag[36][0] = 7; // 7,1
        zigZag[36][1] = 1;
        zigZag[37][0] = 6; // 6,2
        zigZag[37][1] = 2;
        zigZag[38][0] = 5; // 5,3
        zigZag[38][1] = 3;
        zigZag[39][0] = 4; // 4,4
        zigZag[39][1] = 4;
        zigZag[40][0] = 3; // 3,5
        zigZag[40][1] = 5;
        zigZag[41][0] = 2; // 2,6
        zigZag[41][1] = 6;
        zigZag[42][0] = 1; // 1,7
        zigZag[42][1] = 7;
        zigZag[43][0] = 2; // 2,7
        zigZag[43][1] = 7;
        zigZag[44][0] = 3; // 3,6
        zigZag[44][1] = 6;
        zigZag[45][0] = 4; // 4,5
        zigZag[45][1] = 5;
        zigZag[46][0] = 5; // 5,4
        zigZag[46][1] = 4;
        zigZag[47][0] = 6; // 6,3
        zigZag[47][1] = 3;
        zigZag[48][0] = 7; // 7,2
        zigZag[48][1] = 2;
        zigZag[49][0] = 7; // 7,3
        zigZag[49][1] = 3;
        zigZag[50][0] = 6; // 6,4
        zigZag[50][1] = 4;
        zigZag[51][0] = 5; // 5,5
        zigZag[51][1] = 5;
        zigZag[52][0] = 4; // 4,6
        zigZag[52][1] = 6;
        zigZag[53][0] = 3; // 3,7
        zigZag[53][1] = 7;
        zigZag[54][0] = 4; // 4,7
        zigZag[54][1] = 7;
        zigZag[55][0] = 5; // 5,6
        zigZag[55][1] = 6;
        zigZag[56][0] = 6; // 6,5
        zigZag[56][1] = 5;
        zigZag[57][0] = 7; // 7,4
        zigZag[57][1] = 4;
        zigZag[58][0] = 7; // 7,5
        zigZag[58][1] = 5;
        zigZag[59][0] = 6; // 6,6
        zigZag[59][1] = 6;
        zigZag[60][0] = 5; // 5,7
        zigZag[60][1] = 7;
        zigZag[61][0] = 6; // 6,7
        zigZag[61][1] = 7;
        zigZag[62][0] = 7; // 7,6
        zigZag[62][1] = 6;
        zigZag[63][0] = 7; // 7,7
        zigZag[63][1] = 7;
    }

    /*
     * This method preforms a matrix multiplication of the input pixel data matrix
     * by the transposed cosine matrix and store the result in a temporary
     * N * N matrix. This N * N matrix is then multiplied by the cosine matrix
     * and the result is stored in the output matrix.
     *
     * @param input The Input Pixel Matrix
     * @returns output The DCT Result Matrix
     */
    public double[][] forwardDCT(float input[][])
    {
        double output[][] = new double[N][N];
        double temp[][] = new double[N][N];
        double temp1;
        int i;
        int j;
        int k;

        for (i = 0; i < N; i++)
        {
            for (j = 0; j < N; j++)
            {
                temp[i][j] = 0.0;
                for (k = 0; k < N; k++)
                {
                    temp[i][j] += (((input[i][k]) - 128) * cT[k][j]);
                }
            }
        }
        for (i = 0; i < N; i++)
        {
            for (j = 0; j < N; j++)
            {
                temp1 = 0.0;

                for (k = 0; k < N; k++)
                {
                    temp1 += (c[i][k] * temp[k][j]);
                }

                output[i][j] = temp1;
            }
        }

        return output;
    }

    /* This method is preformed using the reverse of the operations preformed in
     * the DCT. This restores a N * N input block to the corresponding output
     * block with values scaled to 0 to 255 and then stored in the input block
     * of pixels.
     *
     * @param input N * N input block
     * @returns output The pixel array output
     */
    public int[][] inverseDCT(double input[][])
    {
        int output[][] = new int[N][N];
        double temp[][] = new double[N][N];
        double temp1;
        int i;
        int j;
        int k;

        for (i=0; i<N; i++)
        {
            for (j=0; j<N; j++)
            {
                temp[i][j] = 0.0;
                for (k=0; k<N; k++)
                {
                    temp[i][j] += input[i][k] * c[k][j];
                }
            }
        }
        for (i=0; i<N; i++)
        {
            for (j=0; j<N; j++)
            {
                temp1 = 0.0;
                for (k=0; k<N; k++)
                {
                    temp1 += cT[i][k] * temp[k][j];
                }
                temp1 += 128.0;
                if (temp1 < 0)
                {
                    output[i][j] = 0;
                }
                else if (temp1 > 255)
                {
                    output[i][j] = 255;
                }
                else
                {
                     output[i][j] = (int)Math.round(temp1);
                }
            }
        }
        return output;
    }



     /*
     * This method orders the DCT result matrix into a zigzag pattern and then
     * quantitizes the data. The quantitized value is rounded to the nearest integer.
     * Pixels which round or divide to zero are the loss associated with
     * quantitizing the image.
     *
     *
     * @param inputData 8x8 array of DCT image data.
     * @param quantum   the quantized matrix associated with the data
     * @returns outputData The quantitized output data
     */
    public int[] quantitizeImage(double inputData[][], float [][] quantum)
    {
        int outputData[] = new int[N*N];

        int i = 0;
        int j = 0;
        int a = 0;
        int b = 0;
        int row;
        int col;
        double result;

         for (i = 0; i < (N*N); i++)
            {
                row = zigZag[i][0];
                col = zigZag[i][1];

                /*
                 * Here, we do a multiplication by 20, in order to improve the quality of the result
                 * It is not something done in the JPEG standart, but it is done to keep more data before
                 * rounding in output. A division must be done in dequantizatzion
                 */
                result =20 * (inputData[row][col]) / (double)(quantum[row][col]);
                outputData[i] = (int)(Math.round(result));
            }
          return outputData;
    }



    /*
     * This method reads in DCT codes  dequanitizes them
     * and places them in the correct location. The codes are stored in the
     * zigzag format so they need to be redirected to a N * N block through
     * simple table lookup. After dequantitization the data needs to be
     * run through an inverse DCT.
     *
     * @param inputData 8x8 Array of quantitized image data
     * @param  quantum  The quantized matrix used for quantization
     * @returns outputData A N * N array of de-quantitized data
     *
     */
    public double[][] dequantitizeImage(int[] inputData,float[][] quantum)
    {
        int i = 0;
        int j = 0;
        int a = 0;
        int b = 0;
        int row;
        int col;
        double outputData[][] = new double[N][N];
        double result;

        for (i=0; i<(N*N); i++)
            {
                row = zigZag[i][0];
                col = zigZag[i][1];
                // Here, a division by 20 is done because a multiplication has been donne in quantification
                result = (double)(inputData[i]) * (double)quantum[row][col]/20;
                outputData[row][col] = result;
            }
        return outputData;
    }

    /* COMPRESSION OF THE SCAN
     * The compression is done on one scan. You must enter the matrix of 8 bits pixels
     * for the component
     * @ param input outputArray[][] = matrix of the scan
                     imageWidth      = width of the image
                     imageHeight     = height of the image
                     code            = 1 for luminance, 0 for chrominance
       @ param output compressedImage= result of the compression
    */
    public int[][] compressImage(float inputArray[][], int imageWidth,int imageHeight, int code)//retour int
    {
        int i = 0;
        int j = 0;
        int a = 0;
        int b = 0;
        int xpos;
        int ypos;
        int dctArray3[]  = new int[64];
        float dctArray1[][] = new float[8][8];
        double dctArray2[][]  = new double[8][8];
        int comp;
        int compressedImage[][] = new int[imageWidth*imageHeight/64][64];
        int place;
        // COMPRESSION OF THE IMAGE

        System.out.println("Initializing compression - DCT & Quantification");
        place=0;
        for (i=0; i<(imageWidth)/8; i++)
        {
            for (j=0; j<(imageHeight)/8; j++)
            {
                xpos = i * 8;
                ypos = j * 8;
                for (a=0; a<8; a++)
                {
                    for (b=0; b<8; b++)
                    {
                        dctArray1[a][b] = inputArray[xpos+a][ypos+b];
                    }
                }
                // Apply the forward DCT to the block
                dctArray2 = forwardDCT(dctArray1);

                // Quantitize
                if (code==1)
                {
                    dctArray3 = quantitizeImage(dctArray2, quantum_luminance);
                }
                else
                {
                    dctArray3 = quantitizeImage(dctArray2, quantum_chrominance);
                }

                // Reconstruct the compressed data in an image...

                for ( a=0; a<64; a++)
                {

                        compressedImage[place][a] = dctArray3[a];
                }
                place=place+1;

            }
        }

        return compressedImage;
    }


    /* DECOMPRESSION OF THE SCAN
     * The decompression is done on one scan. You must enter the matrix of 8 bits pixels
     * for the component
     * @ param input inputArray[][] = matrix of the scan
                     imageWidth      = width of the image
                     imageHeight     = height of the image
                     code            = 1 for luminance, 0 for chrominance
       @ param output compressedImage= result of the compression

     */
    public int[][] decompressImage(int inputArray[][],int imageWidth,int imageHeight, int code)
    {
        int i = 0;
        int j = 0;
        int a = 0;
        int b = 0;
        int xpos;
        int ypos;
        int dctArray1[][] = new int[8][8];
        double dctArray2[][]  = new double[8][8];
        int dctArray3[]  = new int[64];
        int comp;
        int reconstImage[][] = new int[imageWidth][imageHeight];
        int place;
        place=0;
        System.out.println("Initializing Decompression");

        for (i=0; i<(imageWidth)/8; i++)
        {
            for (j=0; j<(imageHeight)/8; j++)
            {

                xpos = i * 8;
                ypos = j * 8;

                for (a=0; a<64; a++)
                {
                        dctArray3[a] = inputArray[place][a];
                }
                place=place+1;
                // Run the dequantitizer
                //dctArray2 = dequantitizeImage(dctArray3);
                if (code==1)
                {
                    dctArray2 = dequantitizeImage(dctArray3, quantum_luminance);
                }
                else
                {
                    dctArray2 = dequantitizeImage(dctArray3, quantum_chrominance);
                }

                // Run the inverse DCT
                dctArray1 = inverseDCT(dctArray2);

                // Overwrite with a new reconstructed image
                //System.out.println(" val= "+dctArray1[0][0]+" avec init"+init[xpos][ypos]);
                for (a=0; a<8; a++)
                {
                    for (b=0; b<8; b++)
                    {
                        reconstImage[xpos+a][ypos+b] = dctArray1[a][b];

                    }
                }
            }
        }
        return reconstImage;
    }

}
