// Copyright (C) 1996 by Florian Raemy & LCAV, Swiss Federal Institute of 
// Technology, Lausanne, Switzerland.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit Florian's Home Page http://studwww.eurecom.fr/~raemy and
// LCAV Department of EPFL: http://lcavwww.epfl.ch

import java.awt.*;
import java.io.*;
import java.util.*;


/*
 *  RLE - A Java implementation of the Run Lenght Encoding
 */


public class RLE
{
    /*
	* The Height of the image
	*/
	public int ImageHeight;

	/*
	* The Width of the image
	*/
	public int ImageWidth;


    /**
    * The constructor of the RLE
    * The width and height of the image are needed for the calculs
    **/
	public RLE(int Width, int Height)
	{
	    ImageHeight=Height;
	    ImageWidth=Width;

	}

   /**
   * RLE is the method providing a Run Lenght encoding after the quantization
   * It works on a zigzag matrix of 64 integers components
   * The output is a stack of RLEdata. It's a LIFO stack where we find on the top the last AC
   * number of zero and amplitude of the zigzag matrix according to RLE and below the DC coefficient
   * @param     Input:      zigzag the zigzag matrix of integers
   *                        prec the precedent DC value of the precedent 64 block. The value is zero if none
   * @returns   Output:     Stack which represent the number of zero and amplitude ( last AC on the top)
   **/

	public Stack RLE_64(int zigzag[],int prec)
	{
	    Stack s=new Stack();
	    RLEdata value;
	    int current[][];    // 0 = nbre zero  1 = amplitude
    	int nbre;
        int i;
        int j;
        int comp;
        current= new int[64][2];

        /*
        * calculate the DC coefficient
        */
        current[0][1]=zigzag[0]-prec;
        current[0][0]=0;


        /*
        * calculate the AC coefficients
        */
        comp=0;
        nbre=1;
        for (i=1;i<64;i++)
        {
            if(i==63)
            {
                if (zigzag[i]==0) // EOB
                {
                     current[nbre][0]=0;
                     current[nbre][1]=0;

                     nbre=nbre+1;
                }
                else
                {
                    current[nbre][0]=comp;
                    current[nbre][1]=zigzag[i];

                    nbre=nbre+1;
                }
            }
            else
            {

                if (zigzag[i]==0)
                {
                    if (comp==15) // ZRL
                    {
                        current[nbre][0]=15;
                        current[nbre][1]=0;
	                    comp=0;

	                    nbre=nbre+1;
	                }
	                else
	                {
	                    comp=comp+1;
	                }
	            }
	            else
	            {
	                current[nbre][0]=comp;
                    current[nbre][1]=zigzag[i];

	                comp=0;
	                nbre=nbre+1;
	            }
	        }
	    }

	    /*
	    * We put on the LIFO stack the elements, for having the AC component on the top of the stack
	    */
	    for (i=0; i<nbre;i++)
	    {
	        value=new RLEdata(current[i][0],current[i][1]);
	        s.push(value);
	    }
	    return s;
	}


	/**
	* Run Lenght Coding Inverse on 64 components
	* From the run lenght data on the stack, it gives the 64 quantized data
	* @param input:         s the stack where the components are put in the order "top = last AC"
	* @param prec:          the value of the previous DC component.
    * @returns output       return the quantized vectors of 64 components placed in the zigzag order
	*/

	public int[] IRLE_64 (Stack s, int prec)
	{
	    RLEdata value[];
	    Object obj;
	    int comp;
	    int i;
	    int output[];
	    int nbre_zero;
	    int nbre_stack;
	    comp=63;
	    output= new int[64];
	    int j;
	    comp=0;
	    value=new RLEdata[64];

	    for (j=0;j<64;j++)
	    {
	        output[j]=0;
	    }

	    /*
	    * Reads the stack
	    */
        nbre_stack=0;
	    while(!s.empty())
	    {
	        obj=s.pop();
	        value[nbre_stack]=(RLEdata)obj;
	        nbre_stack=nbre_stack+1;
	    }

	    for (j=0;j<nbre_stack;j++)
	    {
	        /*
	        * Calculate the DC component
	        */
	        if (j==0)
	        {
	            output[0]=value[nbre_stack-1].amplitude() + prec;
	            comp=1;
	        }

	        /*
	        * Calculate the AC components
	        */
	        else
	        {
	            nbre_zero=value[nbre_stack-j-1].number_zero();
	            for (i=0;i<nbre_zero;i++)
	            {
	                output[comp]=0;
	                comp=comp+1;
	            }
	            output[comp]=value[nbre_stack-j-1].amplitude();
	            comp=comp+1;
	        }
	    }
	    return output;
	}


    /**
    * Run lenght coding on the hole scan
    * @param  input:    input[][] correspond to the quantized matrix
    * @return output:   s[] is a Stack LIFO (See RLE_64), where the components s[0] correspond to the top-left 8x8 blocks
    *                   of the image and the s[max] to the 8x8 blocks of the right-down.
    **/
	public Stack[] RLE(int input[][])
	{
	    Stack[] s;
	    int max;
	    max=ImageWidth*ImageHeight/64;
	    s=new Stack[max];
	    int i;

	    System.out.println("RLE");
	    s[0]=RLE_64(input[0],0);
	    for(i=1;i<max;i++)
	    {
	        s[i]=RLE_64(input[i],input[i-1][0]);
        }
	    return s;
	}


	/**
	* Inverse Run lenght coding on the hole scan
	* @param  input:     s[] is a Stack LIFO (see RLE_64), where the components s[0] correspond to the top-left 8x8 blocks
	*                    of the image and the s[max] to the 8x8 blocks of the right-down.
    * @return output:    a matrix according to the quantized matrix of the scan
	*/
	public int[][] IRLE(Stack input[])
	{
	    int[][] output;
	    int max;
	    max=ImageWidth*ImageHeight/64;
	    output=new int[max][64];
        int i;

        System.out.println("IRLE");
	    output[0]=IRLE_64(input[0],0);
	    for(i=1;i<max;i++)
	    {
	        output[i]=IRLE_64(input[i],output[i-1][0]);
	    }
	    return output;
	}
}
